@@ -5,6 +5,7 @@
 import warnings
 import re
 from .version import version as __version__
+from .version import show_versions
 
 # And all the librosa sub-modules
 from . import cache
@@ -17,6 +18,7 @@ from . import filters
 from . import onset
 from . import output
 from . import segment
+from . import sequence
 from . import util
 
 # Exporting exception classes at the top level
