@@ -20,7 +20,8 @@ Display
 import warnings
 
 import numpy as np
-import matplotlib.pyplot as plt
+from matplotlib.cm import get_cmap
+from matplotlib.axes import Axes
 from matplotlib.ticker import Formatter, ScalarFormatter
 from matplotlib.ticker import LogLocator, FixedLocator, MaxNLocator
 from matplotlib.ticker import SymmetricalLogLocator
@@ -168,7 +169,7 @@ class NoteFormatter(Formatter):
 
         cents = vmax <= 2 * max(1, vmin)
 
-        return core.hz_to_note(int(x), octave=self.octave, cents=cents)[0]
+        return core.hz_to_note(int(x), octave=self.octave, cents=cents)
 
 
 class LogHzFormatter(Formatter):
@@ -303,7 +304,7 @@ def cmap(data, robust=True, cmap_seq='magma', cmap_bool='gray_r', cmap_div='cool
     data = np.atleast_1d(data)
 
     if data.dtype == 'bool':
-        return plt.get_cmap(cmap_bool)
+        return get_cmap(cmap_bool)
 
     data = data[np.isfinite(data)]
 
@@ -316,9 +317,9 @@ def cmap(data, robust=True, cmap_seq='magma', cmap_bool='gray_r', cmap_div='cool
     min_val = np.percentile(data, min_p)
 
     if min_val >= 0 or max_val <= 0:
-        return plt.get_cmap(cmap_seq)
+        return get_cmap(cmap_seq)
 
-    return plt.get_cmap(cmap_div)
+    return get_cmap(cmap_div)
 
 
 def __envelope(x, hop):
@@ -326,8 +327,8 @@ def __envelope(x, hop):
     return util.frame(x, hop_length=hop, frame_length=hop).max(axis=0)
 
 
-def waveplot(y, sr=22050, max_points=5e4, x_axis='time', offset=0.0, max_sr=1000,
-             **kwargs):
+def waveplot(y, sr=22050, max_points=5e4, x_axis='time', offset=0.0,
+             max_sr=1000, ax=None, **kwargs):
     '''Plot the amplitude envelope of a waveform.
 
     If `y` is monophonic, a filled curve is drawn between `[-abs(y), abs(y)]`.
@@ -356,6 +357,9 @@ def waveplot(y, sr=22050, max_points=5e4, x_axis='time', offset=0.0, max_sr=1000
     x_axis : str {'time', 'off', 'none'} or None
         If 'time', the x-axis is given time tick-marks.
 
+    ax : matplotlib.axes.Axes or None
+        Axes to plot on instead of the default `plt.gca()`.
+
     offset : float
         Horizontal offset (in time) to start the waveform plot
 
@@ -435,7 +439,7 @@ def waveplot(y, sr=22050, max_points=5e4, x_axis='time', offset=0.0, max_sr=1000
         y_top = y
         y_bottom = -y
 
-    axes = plt.gca()
+    axes = __check_axes(ax)
 
     kwargs.setdefault('color', next(axes._get_lines.prop_cycler)['color'])
 
@@ -461,6 +465,7 @@ def specshow(data, x_coords=None, y_coords=None,
              sr=22050, hop_length=512,
              fmin=None, fmax=None,
              bins_per_octave=12,
+             ax=None,
              **kwargs):
     '''Display a spectrogram/chromagram/cqt/etc.
 
@@ -542,6 +547,9 @@ def specshow(data, x_coords=None, y_coords=None,
     bins_per_octave : int > 0 [scalar]
         Number of bins per octave.  Used for CQT frequency scale.
 
+    ax : matplotlib.axes.Axes or None
+        Axes to plot on instead of the default `plt.gca()`.
+
     kwargs : additional keyword arguments
         Arguments passed through to `matplotlib.pyplot.pcolormesh`.
 
@@ -653,7 +661,7 @@ def specshow(data, x_coords=None, y_coords=None,
     >>> plt.tight_layout()
     '''
 
-    if np.issubdtype(data.dtype, np.complex):
+    if np.issubdtype(data.dtype, np.complexfloating):
         warnings.warn('Trying to display complex-valued input. '
                       'Showing magnitude instead.')
         data = np.abs(data)
@@ -674,9 +682,9 @@ def specshow(data, x_coords=None, y_coords=None,
     y_coords = __mesh_coords(y_axis, y_coords, data.shape[0], **all_params)
     x_coords = __mesh_coords(x_axis, x_coords, data.shape[1], **all_params)
 
-    axes = plt.gca()
+    axes = __check_axes(ax)
     out = axes.pcolormesh(x_coords, y_coords, data, **kwargs)
-    plt.sci(out)
+    __set_current_image(ax, out)
 
     axes.set_xlim(x_coords.min(), x_coords.max())
     axes.set_ylim(y_coords.min(), y_coords.max())
@@ -692,6 +700,18 @@ def specshow(data, x_coords=None, y_coords=None,
     return axes
 
 
+def __set_current_image(ax, img):
+    '''Helper to set the current image in pyplot mode.
+
+    If the provided `ax` is not `None`, then we assume that the user is using the object API.
+    In this case, the pyplot current image is not set.
+    '''
+
+    if ax is None:
+        import matplotlib.pyplot as plt
+        plt.sci(img)
+
+
 def __mesh_coords(ax_type, coords, n, **kwargs):
     '''Compute axis coordinates'''
 
@@ -723,6 +743,17 @@ def __mesh_coords(ax_type, coords, n, **kwargs):
     return coord_map[ax_type](n, **kwargs)
 
 
+def __check_axes(axes):
+    '''Check if "axes" is an instance of an axis object. If not, use `gca`.'''
+    if axes is None:
+        import matplotlib.pyplot as plt
+        axes = plt.gca()
+    elif not isinstance(axes, Axes):
+        raise ValueError("`axes` must be an instance of matplotlib.axes.Axes. "
+                         "Found type(axes)={}".format(type(axes)))
+    return axes
+
+
 def __scale_axes(axes, ax_type, which):
     '''Set the axis scaling'''
 
@@ -834,7 +865,13 @@ def __decorate_axis(axis, ax_type):
 def __coord_fft_hz(n, sr=22050, **_kwargs):
     '''Get the frequencies for FFT bins'''
     n_fft = 2 * (n - 1)
-    return core.fft_frequencies(sr=sr, n_fft=1+n_fft)
+    # The following code centers the FFT bins at their frequencies
+    # and clips to the non-negative frequency range [0, nyquist]
+    basis = core.fft_frequencies(sr=sr, n_fft=n_fft)
+    fmax = basis[-1]
+    basis -= 0.5 * (basis[1] - basis[0])
+    basis = np.append(np.maximum(0, basis), [fmax])
+    return basis
 
 
 def __coord_mel_hz(n, fmin=0, fmax=11025.0, **_kwargs):
@@ -845,14 +882,21 @@ def __coord_mel_hz(n, fmin=0, fmax=11025.0, **_kwargs):
     if fmax is None:
         fmax = 11025.0
 
-    return core.mel_frequencies(n+2, fmin=fmin, fmax=fmax)[1:]
+    basis = core.mel_frequencies(n, fmin=fmin, fmax=fmax)
+    basis[1:] -= 0.5 * np.diff(basis)
+    basis = np.append(np.maximum(0, basis), [fmax])
+    return basis
 
 
 def __coord_cqt_hz(n, fmin=None, bins_per_octave=12, **_kwargs):
     '''Get CQT bin frequencies'''
     if fmin is None:
         fmin = core.note_to_hz('C1')
-    return core.cqt_frequencies(n+1, fmin=fmin, bins_per_octave=bins_per_octave)
+
+    # we drop by half a bin so that CQT bins are centered vertically
+    return core.cqt_frequencies(n+1,
+                                fmin=fmin / 2.0**(0.5/bins_per_octave),
+                                bins_per_octave=bins_per_octave)
 
 
 def __coord_chroma(n, bins_per_octave=12, **_kwargs):
@@ -862,7 +906,9 @@ def __coord_chroma(n, bins_per_octave=12, **_kwargs):
 
 def __coord_tempo(n, sr=22050, hop_length=512, **_kwargs):
     '''Tempo coordinates'''
-    return core.tempo_frequencies(n+1, sr=sr, hop_length=hop_length)
+    basis = core.tempo_frequencies(n+2, sr=sr, hop_length=hop_length)[1:]
+    edges = np.arange(1, n+2)
+    return basis * (edges + 0.5) / edges
 
 
 def __coord_n(n, **_kwargs):
