@@ -9,10 +9,11 @@ Filter bank construction
 .. autosummary::
     :toctree: generated/
 
-    dct
     mel
     chroma
     constant_q
+    _multirate_fb
+    semitone_filterbank
 
 Window functions
 ----------------
@@ -30,6 +31,15 @@ Miscellaneous
 
     constant_q_lengths
     cq_to_chroma
+    mr_frequencies
+    window_sumsquare
+
+Deprecated
+----------
+.. autosummary::
+    :toctree: generated/
+
+    dct
 """
 import warnings
 
@@ -38,11 +48,14 @@ import scipy
 import scipy.signal
 import six
 
+from numba import jit
+
 from . import cache
 from . import util
 from .util.exceptions import ParameterError
+from .util.decorators import deprecated
 
-from .core.time_frequency import note_to_hz, hz_to_midi, hz_to_octs
+from .core.time_frequency import note_to_hz, hz_to_midi, midi_to_hz, hz_to_octs
 from .core.time_frequency import fft_frequencies, mel_frequencies
 
 __all__ = ['dct',
@@ -52,7 +65,10 @@ __all__ = ['dct',
            'constant_q_lengths',
            'cq_to_chroma',
            'window_bandwidth',
-           'get_window']
+           'get_window',
+           'mr_frequencies',
+           'semitone_filterbank',
+           'window_sumsquare']
 
 
 # Dictionary of window function bandwidths
@@ -99,12 +115,15 @@ WINDOW_BANDWIDTHS = {'bart': 1.3334961334912805,
                      'triangle': 1.3331706523555851}
 
 
-@cache(level=10)
+@deprecated('0.6.1', '0.7.0')
 def dct(n_filters, n_input):
-    """Discrete cosine transform (DCT type-III) basis.
+    """Discrete cosine transform (DCT type-II, normalized) basis.
 
     .. [1] http://en.wikipedia.org/wiki/Discrete_cosine_transform
 
+    .. warning:: This function is deprecated in librosa 0.6.1. It will
+        be removed in 0.7.0.
+
     Parameters
     ----------
     n_filters : int > 0 [scalar]
@@ -116,12 +135,16 @@ def dct(n_filters, n_input):
     Returns
     -------
     dct_basis: np.ndarray [shape=(n_filters, n_input)]
-        DCT (type-III) basis vectors [1]_
+        DCT (type-II) basis vectors [1]_
 
     Notes
     -----
     This function caches at level 10.
 
+    See Also
+    --------
+    scipy.fftpack.dct
+
     Examples
     --------
     >>> n_fft = 2048
@@ -180,8 +203,8 @@ def mel(sr, n_fft, n_mels=128, fmin=0.0, fmax=None, htk=False,
         use HTK formula instead of Slaney
 
     norm : {None, 1, np.inf} [scalar]
-        if 1, divide the triangular mel weights by the width of the mel band 
-        (area normalization).  Otherwise, leave all the triangles aiming for 
+        if 1, divide the triangular mel weights by the width of the mel band
+        (area normalization).  Otherwise, leave all the triangles aiming for
         a peak value of 1.0
 
     Returns
@@ -553,10 +576,10 @@ def constant_q(sr, fmin=None, n_bins=84, bins_per_octave=12, tuning=0.0,
     filters = []
     for ilen, freq in zip(lengths, freqs):
         # Build the filter: note, length will be ceil(ilen)
-        sig = np.exp(np.arange(ilen, dtype=float) * 1j * 2 * np.pi * freq / sr)
+        sig = np.exp(np.arange(-ilen//2, ilen//2, dtype=float) * 1j * 2 * np.pi * freq / sr)
 
         # Apply the windowing function
-        sig = sig * __float_window(window)(ilen)
+        sig = sig * __float_window(window)(len(sig))
 
         # Normalize
         sig = util.normalize(sig, norm=norm)
@@ -882,3 +905,305 @@ def get_window(window, Nx, fftbins=True):
                              '{:d} != {:d}'.format(len(window), Nx))
     else:
         raise ParameterError('Invalid window specification: {}'.format(window))
+
+
+@cache(level=10)
+def _multirate_fb(center_freqs=None, sample_rates=None, Q=25.0,
+                  passband_ripple=1, stopband_attenuation=50, ftype='ellip'):
+    r'''Helper function to construct a multirate filterbank.
+
+     A filter bank consists of multiple band-pass filters which divide the input signal
+     into subbands. In the case of a multirate filter bank, the band-pass filters
+     operate with resampled versions of the input signal, e.g. to keep the length
+     of a filter constant while shifting its center frequency.
+
+     This implementation uses `scipy.signal.iirdesign` to design the filters.
+
+
+    Parameters
+    ----------
+    center_freqs : np.ndarray [shape=(n,), dtype=float]
+        Center frequencies of the filter kernels.
+        Also defines the number of filters in the filterbank.
+
+    sample_rates : np.ndarray [shape=(n,), dtype=float]
+        Samplerate for each filter (used for multirate filterbank).
+
+    Q : float
+        Q factor (influences the filter bandwith).
+
+    passband_ripple : float
+        The maximum loss in the passband (dB)
+        See `scipy.signal.iirdesign` for details.
+
+    stopband_attenuation : float
+        The minimum attenuation in the stopband (dB)
+        See `scipy.signal.iirdesign` for details.
+
+    ftype : str
+        The type of IIR filter to design
+        See `scipy.signal.iirdesign` for details.
+
+    Returns
+    -------
+    filterbank : list [shape=(n,), dtype=float]
+        Each list entry comprises the filter coefficients for a single filter.
+
+    sample_rates : np.ndarray [shape=(n,), dtype=float]
+        Samplerate for each filter.
+
+    Notes
+    -----
+    This function caches at level 10.
+
+    See Also
+    --------
+    scipy.signal.iirdesign
+
+    Raises
+    ------
+    ParameterError
+        If `center_freqs` is `None`.
+        If `sample_rates` is `None`.
+        If `center_freqs.shape` does not match `sample_rates.shape`.
+    '''
+
+    if center_freqs is None:
+        raise ParameterError('center_freqs must be provided.')
+
+    if sample_rates is None:
+        raise ParameterError('sample_rates must be provided.')
+
+    if center_freqs.shape != sample_rates.shape:
+        raise ParameterError('Number of provided center_freqs and sample_rates must be equal.')
+
+    nyquist = 0.5 * sample_rates
+    filter_bandwidths = center_freqs / float(Q)
+
+    filterbank = []
+
+    for cur_center_freq, cur_nyquist, cur_bw in zip(center_freqs, nyquist, filter_bandwidths):
+        passband_freqs = [cur_center_freq - 0.5 * cur_bw, cur_center_freq + 0.5 * cur_bw] / cur_nyquist
+        stopband_freqs = [cur_center_freq - cur_bw, cur_center_freq + cur_bw] / cur_nyquist
+
+        cur_filter = scipy.signal.iirdesign(passband_freqs, stopband_freqs,
+                                            passband_ripple, stopband_attenuation,
+                                            analog=False, ftype=ftype, output='ba')
+
+        filterbank.append(cur_filter)
+
+    return filterbank, sample_rates
+
+
+@cache(level=10)
+def mr_frequencies(tuning):
+    r'''Helper function for generating center frequency and sample rate pairs.
+
+    This function will return center frequency and corresponding sample rates
+    to obtain similar pitch filterbank settings as described in [1]_.
+    Instead of starting with MIDI pitch `A0`, we start with `C0`.
+
+    .. [1] Müller, Meinard.
+           "Information Retrieval for Music and Motion."
+           Springer Verlag. 2007.
+
+
+    Parameters
+    ----------
+    tuning : float in `[-0.5, +0.5)` [scalar]
+        Tuning deviation from A440, measure as a fraction of the equally
+        tempered semitone (1/12 of an octave).
+
+    Returns
+    -------
+    center_freqs : np.ndarray [shape=(n,), dtype=float]
+        Center frequencies of the filter kernels.
+        Also defines the number of filters in the filterbank.
+
+    sample_rates : np.ndarray [shape=(n,), dtype=float]
+        Sample rate for each filter, used for multirate filterbank.
+
+    Notes
+    -----
+    This function caches at level 10.
+
+
+    See Also
+    --------
+    librosa.filters.semitone_filterbank
+    librosa.filters._multirate_fb
+    '''
+
+    center_freqs = midi_to_hz(np.arange(24 + tuning, 109 + tuning))
+
+    sample_rates = np.asarray(len(np.arange(0, 36)) * [882, ] +
+                              len(np.arange(36, 70)) * [4410, ] +
+                              len(np.arange(70, 85)) * [22050, ])
+
+    return center_freqs, sample_rates
+
+
+def semitone_filterbank(center_freqs=None, tuning=0.0, sample_rates=None, **kwargs):
+    r'''Constructs a multirate filterbank of infinite-impulse response (IIR)
+    band-pass filters at user-defined center frequencies and sample rates.
+
+    By default, these center frequencies are set equal to the 88 fundamental
+    frequencies of the grand piano keyboard, according to a pitch tuning standard
+    of A440, that is, note A above middle C set to 440 Hz. The center frequencies
+    are tuned to the twelve-tone equal temperament, which means that they grow
+    exponentially at a rate of 2**(1/12), that is, twelve notes per octave.
+
+    The A440 tuning can be changed by the user while keeping twelve-tone equal
+    temperament. While A440 is currently the international standard in the music
+    industry (ISO 16), some orchestras tune to A441-A445, whereas baroque musicians
+    tune to A415.
+
+    See [1]_ for details.
+
+    .. [1] Müller, Meinard.
+           "Information Retrieval for Music and Motion."
+           Springer Verlag. 2007.
+
+
+    Parameters
+    ----------
+    center_freqs : np.ndarray [shape=(n,), dtype=float]
+        Center frequencies of the filter kernels.
+        Also defines the number of filters in the filterbank.
+
+    tuning : float in `[-0.5, +0.5)` [scalar]
+        Tuning deviation from A440 as a fraction of a semitone (1/12 of an octave
+        in equal temperament).
+
+    sample_rates : np.ndarray [shape=(n,), dtype=float]
+        Sample rates of each filter in the multirate filterbank.
+
+    kwargs : additional keyword arguments
+        Additional arguments to the private function `_multirate_fb()`.
+
+    Returns
+    -------
+    filterbank : list [shape=(n,), dtype=float]
+        Each list entry contains the filter coefficients for a single filter.
+
+    fb_sample_rates : np.ndarray [shape=(n,), dtype=float]
+        Sample rate for each filter.
+
+    See Also
+    --------
+    librosa.core.cqt
+    librosa.core.iirt
+    librosa.filters._multirate_fb
+    librosa.filters.mr_frequencies
+    scipy.signal.iirdesign
+
+    Examples
+    --------
+    >>> import matplotlib.pyplot as plt
+    >>> import numpy as np
+    >>> import scipy.signal
+    >>> semitone_filterbank, sample_rates = librosa.filters.semitone_filterbank()
+    >>> plt.figure(figsize=(10, 6))
+    >>> for cur_sr, cur_filter in zip(sample_rates, semitone_filterbank):
+    ...    w, h = scipy.signal.freqz(cur_filter[0], cur_filter[1], worN=2000)
+    ...    plt.plot((cur_sr / (2 * np.pi)) * w, 20 * np.log10(abs(h)))
+    >>> plt.semilogx()
+    >>> plt.xlim([20, 10e3])
+    >>> plt.ylim([-60, 3])
+    >>> plt.title('Magnitude Responses of the Pitch Filterbank')
+    >>> plt.xlabel('Log-Frequency (Hz)')
+    >>> plt.ylabel('Magnitude (dB)')
+    >>> plt.tight_layout()
+    '''
+
+    if (center_freqs is None) and (sample_rates is None):
+        center_freqs, sample_rates = mr_frequencies(tuning)
+
+    filterbank, fb_sample_rates = _multirate_fb(center_freqs=center_freqs, sample_rates=sample_rates, **kwargs)
+
+    return filterbank, fb_sample_rates
+
+
+@jit(nopython=True)
+def __window_ss_fill(x, win_sq, n_frames, hop_length):  # pragma: no cover
+    '''Helper function for window sum-square calculation.'''
+
+    n = len(x)
+    n_fft = len(win_sq)
+    for i in range(n_frames):
+        sample = i * hop_length
+        x[sample:min(n, sample + n_fft)] += win_sq[:max(0, min(n_fft, n - sample))]
+
+
+def window_sumsquare(window, n_frames, hop_length=512, win_length=None, n_fft=2048,
+                     dtype=np.float32, norm=None):
+    '''
+    Compute the sum-square envelope of a window function at a given hop length.
+
+    This is used to estimate modulation effects induced by windowing observations
+    in short-time fourier transforms.
+
+    Parameters
+    ----------
+    window : string, tuple, number, callable, or list-like
+        Window specification, as in `get_window`
+
+    n_frames : int > 0
+        The number of analysis frames
+
+    hop_length : int > 0
+        The number of samples to advance between frames
+
+    win_length : [optional]
+        The length of the window function.  By default, this matches `n_fft`.
+
+    n_fft : int > 0
+        The length of each analysis frame.
+
+    dtype : np.dtype
+        The data type of the output
+
+    Returns
+    -------
+    wss : np.ndarray, shape=`(n_fft + hop_length * (n_frames - 1))`
+        The sum-squared envelope of the window function
+
+    Examples
+    --------
+    For a fixed frame length (2048), compare modulation effects for a Hann window
+    at different hop lengths:
+
+    >>> n_frames = 50
+    >>> wss_256 = librosa.filters.window_sumsquare('hann', n_frames, hop_length=256)
+    >>> wss_512 = librosa.filters.window_sumsquare('hann', n_frames, hop_length=512)
+    >>> wss_1024 = librosa.filters.window_sumsquare('hann', n_frames, hop_length=1024)
+
+    >>> import matplotlib.pyplot as plt
+    >>> plt.figure()
+    >>> plt.subplot(3,1,1)
+    >>> plt.plot(wss_256)
+    >>> plt.title('hop_length=256')
+    >>> plt.subplot(3,1,2)
+    >>> plt.plot(wss_512)
+    >>> plt.title('hop_length=512')
+    >>> plt.subplot(3,1,3)
+    >>> plt.plot(wss_1024)
+    >>> plt.title('hop_length=1024')
+    >>> plt.tight_layout()
+
+    '''
+    if win_length is None:
+        win_length = n_fft
+
+    n = n_fft + hop_length * (n_frames - 1)
+    x = np.zeros(n, dtype=dtype)
+
+    # Compute the squared window at the desired length
+    win_sq = get_window(window, win_length)
+    win_sq = util.normalize(win_sq, norm=norm)**2
+    win_sq = util.pad_center(win_sq, n_fft)
+
+    # Fill the envelope
+    __window_ss_fill(x, win_sq, n_frames, hop_length)
+
+    return x
